import pgmagick


def _convert_colorobj(input_obj):
    if isinstance(input_obj, (list, tuple)):
        r, g, b = int(input_obj[0]), int(input_obj[1]), int(input_obj[2])
        color = pgmagick.Color(r, g, b)
    elif isinstance(input_obj, str):
        color = pgmagick.Color(input_obj)
    else:
        color = input_obj
        assert isinstance(color, pgmagick.Color)
    return color


def _convert_coordinatelist(input_obj):
    """convert from 'list' or 'tuple' object to pgmagick.CoordinateList.

    :type input_obj: list or tuple
    """
    cdl = pgmagick.CoordinateList()
    for obj in input_obj:
        cdl.append(pgmagick.Coordinate(obj[0], obj[1]))
    return cdl


def _convert_paintmethod(input_obj):
    if isinstance(input_obj, pgmagick.PaintMethod):
        return input_obj
    pm = pgmagick.PaintMethod()
    if input_obj.lower() == 'filltoborder':
        paint_method = pm.FillToBorderMethod
    else:
        paint_method = getattr(pm, "%sMethod" % input_obj.title())
    return paint_method


def _convert_vpathlist(input_obj):
    """convert from 'list' or 'tuple' object to pgmagick.VPathList.

    :type input_obj: list or tuple
    """
    vpl = pgmagick.VPathList()
    for obj in input_obj:
        # FIXME
        obj = pgmagick.PathMovetoAbs(pgmagick.Coordinate(obj[0], obj[1]))
        vpl.append(obj)
    return vpl


class Image(object):

    @property
    def height(self):
        return self.img.rows()

    @property
    def width(self):
        return self.img.columns()

    def __init__(self, filename=None, color=None, *args, **kargs):
        self.img = None
        if isinstance(filename, (unicode, str)):
            self.img = pgmagick.Image(str(filename))
        elif isinstance(filename, (list, tuple)):
            size = filename
            geometry = pgmagick.Geometry(int(size[0]), int(size[1]))
            if isinstance(color, (list, tuple)):
                r, g, b = int(color[0]), int(color[1]), int(color[2])
                color = pgmagick.Color(r, g, b)
                self.img = pgmagick.Image(geometry, color)
            elif isinstance(color, str):
                if color.find('gradient') == 0 or color.find('plasma') == 0:
                    self.img = pgmagick.Image(geometry, pgmagick.Color())
                    self.img.read(color)
                else:
                    color = pgmagick.Color(color)
                    self.img = pgmagick.Image(geometry, color)
            else:
                self.img = pgmagick.Image(geometry, pgmagick.Color())
        else:
            self.img = pgmagick.Image()

    def write(self, filename):
        self.img.write(str(filename))

    # API of Manipulate An Image
    def adaptive_threshold(self, width, height, offset=0):
        # TODO: not implemented
        pass

    def add_noise(self, noise_type):
        # TODO: not implemented
        pass

    def add_noise_channel(self, channel, noise_type):
        # TODO: not implemented
        pass

    def affine_transform(self, affine):
        # TODO: not implemented
        pass

    def annotate(self, string, position=(0, 0), gravity='center', angle=0):
        position = pgmagick.Geometry(int(position[0]), int(position[1]))
        gravity = getattr(pgmagick.GravityType, "%sGravity" % gravity.title())
        self.img.annotate(string, position, gravity, angle)

    def blur(self, radius=0.0, sigma=1.0):
        # TODO: not implemented
        pass

    def blur_channel(self, channel, radius=0.0, sigma=1.0):
        if isinstance(channel, str):
            channel = getattr(pgmagick.ChannelType,
                              "%sChannel" % channel.title())
        self.img.blurChannel(channel, radius, sigma)

    def border(self, geometry=None):
        # TODO: not implemented
        pass

    def channel(self, channel):
        # TODO: not implemented
        pass

    def channel_depth(self, channel, depth=None):
        # TODO: not implemented
        pass

    def charcoal(self, radius=0.0, sigma=1.0):
        # TODO: not implemented
        pass

    def chop(self, geometry):
        # TODO: not implemented
        pass

    def colorize(self, opacity_rgb, color):
        # TODO: not implemented
        pass

    def color_matrix(self, order, color_matrix):
        # TODO: not implemented
        pass

    def comment(self, comment):
        # TODO: not implemented
        pass

    def compare(self, reference):
        # TODO: not implemented
        pass

    def composite(self, composite_img, offset,
                  compose=pgmagick.CompositeOperator.InCompositeOp):
        if isinstance(offset, (list, tuple)):
            x = int(offset[0])
            y = int(offset[1])
            offset = pgmagick.Geometry(x, y)
        elif isinstance(offset, pgmagick.Geometry):
            pass
        elif isinstance(offset, str):   # is gravity (string)
            offset = getattr(pgmagick.GravityType,
                             "%sGravity" % offset.title())
        else:   # is gravity (pgmagick.GravityType)
            pass
        if isinstance(compose, pgmagick.CompositeOperator):
            pass
        elif compose.lower() in ('copyblue', 'copygreen', 'copyopacity',
                                 'copyred', 'copycyan', 'copymagenta',
                                 'copyyellow', 'copyblack'):
            color = compose.lower().split('copy')[1].title()
            compose = getattr(pgmagick.CompositeOperator,
                              "Copy%sCompositeOp" % color)
        else:   # other string
            compose = getattr(pgmagick.CompositeOperator,
                              "%sCompositeOp" % compose.title())
        self.img.composite(composite_img, offset, compose)

    def contrast(self, sharpen):
        # TODO: not implemented
        pass

    def convolve(self, order, kernel):
        # TODO: not implemented
        pass

    def crop(self, *geometry):
        if len(geometry) == 4:
            width, height = geometry[0], geometry[1]
            x, y = geometry[2], geometry[3]
            g = pgmagick.Geometry(x, y, width, height)
        elif isinstance(geometry, pgmagick.Geometry):
            g = geometry
        else:
            raise Exception("not support object", geometry)
        self.img.crop(g)

    def cycle_colormap(self, amount):
        # TODO: not implemented
        pass

    def draw(self, draw_obj):
        if isinstance(draw_obj, (list, tuple)):
            draw = pgmagick.DrawableList()
            for d in draw_obj:
                draw.append(d)
        elif isinstance(draw_obj, Draw):
            draw = draw_obj.drawer
        else:
            draw = draw_obj
        self.img.draw(draw)

    def edge(self, radius=0.0):
        # TODO: not implemented
        pass

    def emboss(self, radius=0.0, sigma=1.0):
        # TODO: not implemented
        pass

    def floodfill_color(self, point, fill_color):
        # TODO: not implemented
        pass

    def floodfill_opacity(self, point, opacity, method):
        # TODO: not implemented
        pass

    def floodfill_texture(self, point, texture):
        # TODO: not implemented
        pass

    def scale(self, size, filter_type=None):
        if isinstance(size, float):
            scaled_height = self.img.rows() * size
            scaled_width = self.img.columns() * size
            size = "%dx%d" % (int(scaled_width), int(scaled_height))
        elif isinstance(size, (list, tuple)):
            scaled_width, scaled_height = int(size[0]), int(size[1])
            size = "%dx%d" % (int(scaled_width), int(scaled_height))
        if filter_type:
            filter_type = getattr(pgmagick.FilterTypes,
                                  "%sFilter" % filter_type.title())
            pgmagick.Image.filterType(self, filter_type)
        geometry = pgmagick.Geometry(size)
        self.img.scale(geometry)

    # API of Set/Get Image
    def font_pointsize(self, point_size=None):
        if point_size:
            point_size = float(point_size)
            self.img.fontPointsize(point_size)
        else:
            return self.img.fontPointsize()


class Draw(object):

    def __init__(self):
        self.drawer = pgmagick.DrawableList()

    def affine(self, sx, sy, rx, ry, tx, ty):
        affine = pgmagick.DrawableAffine(float(sx), float(sy),
                                         float(rx), float(ry),
                                         float(tx), float(ty))
        self.drawer.append(affine)

    def arc(self, start_x, start_y, end_x, end_y, start_degrees, end_degrees):
        arc = pgmagick.DrawableArc(float(start_x), float(start_y),
                                   float(end_x), float(end_y),
                                   float(start_degrees), float(end_degrees))
        self.drawer.append(arc)

    def bezier(self, points):
        """Draw a Bezier-curve.

        :param points: ex.) ((5, 5), (6, 6), (7, 7))
        :type points: list
        """
        coordinates = pgmagick.CoordinateList()
        for point in points:
            x, y = float(point[0]), float(point[1])
            coordinates.append(pgmagick.Coordinate(x, y))
        self.drawer.append(pgmagick.DrawableBezier(coordinates))

    def circle(self, origin_x, origin_y, perim_x, perim_y):
        circle = pgmagick.DrawableCircle(float(origin_x), float(origin_y),
                                         float(perim_x), float(perim_y))
        self.drawer.append(circle)

    def color(self, x, y, paint_method):
        """
        :param paint_method: 'point' or 'replace' or 'floodfill' or
                             'filltoborder' or 'reset'
        :type paint_method: str or pgmagick.PaintMethod
        """
        paint_method = _convert_paintmethod(paint_method)
        color = pgmagick.DrawableColor(x, y, paint_method)
        self.drawer.append(color)

    def composite(self, x, y, width, height, image,
                  op=pgmagick.CompositeOperator.OverCompositeOp):
        # FIXME: unable to composite pgmagick.Image object.
        if width == 0 or height == 0:
            composite = pgmagick.DrawableCompositeImage(x, y, image)
        else:
            composite = pgmagick.DrawableCompositeImage(x, y, width, height,
                                                        image, op)
        self.drawer.append(composite)

    def ellipse(self, org_x, org_y, radius_x, radius_y, arc_start, arc_end):
        """
        :param org_x: origination x axis
        :param org_y: origination y axis
        :param radius_x: radius x axis
        :param radius_y: radius y axis
        :param arc_start: arc start angle
        :param arc_end: arc end angle
        """
        ellipse = pgmagick.DrawableEllipse(float(org_x), float(org_y),
                                           float(radius_x), float(radius_y),
                                           float(arc_start), float(arc_end))
        self.drawer.append(ellipse)

    def fill_color(self, color):
        color = _convert_colorobj(color)
        fill_color = pgmagick.DrawableFillColor(color)
        self.drawer.append(fill_color)

    def fill_rule(self, rule='evenodd'):
        if rule.lower() == 'evenodd':
            fill_rule = pgmagick.FillRule.EvenOddRule
        elif rule.lower() == 'nonzero':
            fill_rule = pgmagick.FillRule.NonZeroRule
        else:
            fill_rule = rule
        fill_rule = pgmagick.DrawableFillRule(fill_rule)
        self.drawer.append(fill_rule)

    def fill_opacity(self, opacity):
        """
        :param opacity: 0.0 ~ 1.0
        """
        opacity = pgmagick.DrawableFillOpacity(float(opacity))
        self.drawer.append(opacity)

    def font(self, family, style='normal', weight=400, stretch='normal'):
        style = getattr(pgmagick.StyleType, "%sStyle" % style.title())
        stretch = stretch.lower()
        if 'condensed' in stretch:
            tmp = stretch.split('condensed')[0]
            stretch = "%sCondensedStretch" % tmp.title()
        elif 'expanded' in stretch:
            tmp = stretch.split('expanded')[0]
            stretch = "%sExpandedStretch" % tmp.title()
        else:
            stretch = "%sStretch" % stretch.title()
        stretch = getattr(pgmagick.StretchType, "%s" % stretch)
        if weight is 'bold':
            weight = 800
        font = pgmagick.DrawableFont(family, style, weight, stretch)
        self.drawer.append(font)

    def gravity(self, gravity_type):
        if isinstance(gravity_type, str):
            gravity = getattr(pgmagick.GravityType,
                              "%sGravity" % gravity_type.title())
        else:
            gravity = gravity_type
        gravity_type = pgmagick.DrawableGravity(gravity)
        self.drawer.append(gravity_type)

    def line(self, start_x, start_y, end_x, end_y):
        line = pgmagick.DrawableLine(start_x, start_y, end_x, end_y)
        self.drawer.append(line)

    def matte(self, x, y, paint_method):
        """
        :param paint_method: 'point' or 'replace' or 'floodfill' or
                             'filltoborder' or 'reset'
        :type paint_method: str or pgmagick.PaintMethod
        """
        paint_method = _convert_paintmethod(paint_method)
        self.drawer.append(pgmagick.DrawableMatte(x, y, paint_method))

    def miterlimit(self, miterlimit):
        # FIXME
        self.drawer.append(pgmagick.DrawableMiterLimit(miterlimit))

    def path(self, vpath):
        # FIXME
        if isinstance(vpath, pgmagick.VPathList):
            vpath = vpath
        elif isinstance(vpath, (list, tuple)):
            vpath = _convert_vpathlist(vpath)
        self.drawer.append(pgmagick.DrawablePath(vpath))

    def point(self, x, y):
        self.drawer.append(pgmagick.DrawablePoint(x, y))

    def pointsize(self, pointsize):
        pointsize = pgmagick.DrawablePointSize(pointsize)
        self.drawer.append(pointsize)

    def polygon(self, coordinates):
        if isinstance(coordinates, pgmagick.CoordinateList):
            obj = coordinates
        elif isinstance(coordinates, (list, tuple)):
            obj = _convert_coordinatelist(coordinates)
        self.drawer.append(pgmagick.DrawablePolygon(obj))

    def polyline(self, coordinates):
        if isinstance(coordinates, pgmagick.CoordinateList):
            obj = coordinates
        elif isinstance(coordinates, (list, tuple)):
            obj = _convert_coordinatelist(coordinates)
        self.drawer.append(pgmagick.DrawablePolyline(obj))

    def rectangle(self, upperleft_x, upperleft_y, lowerright_x, lowerright_y):
        r = pgmagick.DrawableRectangle(upperleft_x, upperleft_y,
                                       lowerright_x, lowerright_y)
        self.drawer.append(r)

    def rotation(self, angle):
        self.drawer.append(pgmagick.DrawableRotation(angle))

    def round_rectangle(self, center_x, center_y,
                        width, height, corner_width, corner_height):
        rr = pgmagick.DrawableRoundRectangle(center_x, center_y,
                                             width, height,
                                             corner_width, corner_height)
        self.drawer.append(rr)

    def scaling(self, x, y):
        """Scaling Draw Object

        :param x: 0.0 ~ 1.0
        :param y: 0.0 ~ 1.0
        """
        self.drawer.append(pgmagick.DrawableScaling(float(x), float(y)))

    def skewx(self, angle):
        self.drawer.append(pgmagick.DrawableSkewX(float(angle)))

    def skewy(self, angle):
        self.drawer.append(pgmagick.DrawableSkewY(float(angle)))

    def stroke_antialias(self, flag=True):
        """stroke antialias

        :param flag: True or False. (default is True)
        :type flag: bool
        """
        antialias = pgmagick.DrawableStrokeAntialias(flag)
        self.drawer.append(antialias)

    def stroke_color(self, color):
        color = _convert_colorobj(color)
        color = pgmagick.DrawableStrokeColor(color)
        self.drawer.append(color)

    def stroke_linecap(self, linecap):
        """set to stroke linecap.

        :param linecap: 'undefined', 'butt', 'round', 'square'
        :type linecap: str
        """
        linecap = getattr(pgmagick.LineCap, "%sCap" % linecap.title())
        linecap = pgmagick.DrawableStrokeLineCap(linecap)
        self.drawer.append(linecap)

    def stroke_linejoin(self, linejoin):
        """set to stroke linejoin.

        :param linejoin: 'undefined', 'miter', 'round', 'bevel'
        :type linejoin: str
        """
        linejoin = getattr(pgmagick.LineJoin, "%sJoin" % linejoin.title())
        linejoin = pgmagick.DrawableStrokeLineJoin(linejoin)
        self.drawer.append(linejoin)

    def stroke_opacity(self, opacity):
        self.drawer.append(pgmagick.DrawableStrokeOpacity(float(opacity)))

    def stroke_width(self, width):
        width = float(width)
        width = pgmagick.DrawableStrokeWidth(width)
        self.drawer.append(width)

    def text(self, x, y, string, encoding=None):
        # TODO: unable encoding
        text = pgmagick.DrawableText(x, y, string)
        self.drawer.append(text)

    def text_antialias(self, flag=True):
        """text antialias

        :param flag: True or False. (default is True)
        :type flag: bool
        """
        antialias = pgmagick.DrawableTextAntialias(flag)
        self.drawer.append(antialias)

    def text_decoration(self, decoration):
        """text decoration

        :param decoration: 'no', 'underline', 'overline', 'linethrough'
        :type decoration: str
        """
        if decoration.lower() == 'linethrough':
            d = pgmagick.DecorationType.LineThroughDecoration
        else:
            decoration_type_string = "%sDecoration" % decoration.title()
            d = getattr(pgmagick.DecorationType, "%s" % decoration_type_string)
        decoration = pgmagick.DrawableTextDecoration(d)
        self.drawer.append(decoration)

    def text_undercolor(self, color):
        color = _convert_colorobj(color)
        undercolor = pgmagick.DrawableTextUnderColor(color)
        self.drawer.append(undercolor)

    def translate(self, x, y):
        self.drawer.append(pgmagick.DrawableTranslation(float(x), float(y)))



import _pgmagick
__version__ = '0.3.4'


def __init():
    _pgmagick.InitializeMagick("./")
__init()


class Blob(_pgmagick.Blob):

    def __init__(self, *args):
        if len(args) == 1 and isinstance(args[0], str):
            _pgmagick.Blob.__init__(self)
            self.update(args[0])
        else:
            _pgmagick.Blob.__init__(self, *args)

    @property
    def data(self):
        return _pgmagick.get_blob_data(self)

    @data.setter
    def data(self, string):
        self.update(string)

ChannelType = _pgmagick.ChannelType

class Color(_pgmagick.Color):
    pass

class ColorGray(_pgmagick.ColorGray):
    pass

class ColorHSL(_pgmagick.ColorHSL):
    pass

class ColorMono(_pgmagick.ColorMono):
    pass

class ColorRGB(_pgmagick.ColorRGB):
    pass

class ColorYUV(_pgmagick.ColorYUV):
    pass

ColorspaceType = _pgmagick.ColorspaceType

CompositeOperator = _pgmagick.CompositeOperator

CompressionType = _pgmagick.CompressionType

class Coordinate(_pgmagick.Coordinate):
    pass

class CoordinateList(_pgmagick.CoordinateList):
    pass

DecorationType = _pgmagick.DecorationType

class Drawable(_pgmagick.Drawable):
    pass

class DrawableAffine(_pgmagick.DrawableAffine):
    pass

class DrawableArc(_pgmagick.DrawableArc):
    pass

class DrawableBezier(_pgmagick.DrawableBezier):
    pass

class DrawableCircle(_pgmagick.DrawableCircle):
    pass

class DrawableClipPath(_pgmagick.DrawableClipPath):
    pass

class DrawableColor(_pgmagick.DrawableColor):
    pass

class DrawableCompositeImage(_pgmagick.DrawableCompositeImage):
    pass

class DrawableDashArray(_pgmagick.DrawableDashArray):
    pass

class DrawableDashOffset(_pgmagick.DrawableDashOffset):
    pass

class DrawableEllipse(_pgmagick.DrawableEllipse):
    pass

class DrawableFillColor(_pgmagick.DrawableFillColor):
    pass

class DrawableFillOpacity(_pgmagick.DrawableFillOpacity):
    pass

class DrawableFillRule(_pgmagick.DrawableFillRule):
    pass

class DrawableFont(_pgmagick.DrawableFont):
    pass

class DrawableGravity(_pgmagick.DrawableGravity):
    pass

class DrawableLine(_pgmagick.DrawableLine):
    pass

class DrawableList(_pgmagick.DrawableList):
    pass

class DrawableMatte(_pgmagick.DrawableMatte):
    pass

class DrawableMiterLimit(_pgmagick.DrawableMiterLimit):
    pass

class DrawablePath(_pgmagick.DrawablePath):
    pass

class DrawablePoint(_pgmagick.DrawablePoint):
    pass

class DrawablePointSize(_pgmagick.DrawablePointSize):
    pass

class DrawablePolygon(_pgmagick.DrawablePolygon):
    pass

class DrawablePolyline(_pgmagick.DrawablePolyline):
    pass

class DrawablePopClipPath(_pgmagick.DrawablePopClipPath):
    pass

class DrawablePopGraphicContext(_pgmagick.DrawablePopGraphicContext):
    pass

class DrawablePopPattern(_pgmagick.DrawablePopPattern):
    pass

class DrawablePushClipPath(_pgmagick.DrawablePushClipPath):
    pass

class DrawablePushGraphicContext(_pgmagick.DrawablePushGraphicContext):
    pass

class DrawablePushPattern(_pgmagick.DrawablePushPattern):
    pass

class DrawableRectangle(_pgmagick.DrawableRectangle):
    pass

class DrawableRotation(_pgmagick.DrawableRotation):
    pass

class DrawableRoundRectangle(_pgmagick.DrawableRoundRectangle):
    pass

class DrawableScaling(_pgmagick.DrawableScaling):
    pass

class DrawableSkewX(_pgmagick.DrawableSkewX):
    pass

class DrawableSkewY(_pgmagick.DrawableSkewY):
    pass

class DrawableStrokeAntialias(_pgmagick.DrawableStrokeAntialias):
    pass

class DrawableStrokeColor(_pgmagick.DrawableStrokeColor):
    pass

class DrawableStrokeLineCap(_pgmagick.DrawableStrokeLineCap):
    pass

class DrawableStrokeLineJoin(_pgmagick.DrawableStrokeLineJoin):
    pass

class DrawableStrokeOpacity(_pgmagick.DrawableStrokeOpacity):
    pass

class DrawableStrokeWidth(_pgmagick.DrawableStrokeWidth):
    pass

class DrawableText(_pgmagick.DrawableText):
    pass

class DrawableTextAntialias(_pgmagick.DrawableTextAntialias):
    pass

class DrawableTextDecoration(_pgmagick.DrawableTextDecoration):
    pass

class DrawableTextUnderColor(_pgmagick.DrawableTextUnderColor):
    pass

class DrawableTranslation(_pgmagick.DrawableTranslation):
    pass

class DrawableViewbox(_pgmagick.DrawableViewbox):
    pass

EndianType = _pgmagick.EndianType

class Exception(_pgmagick.Exception):
    pass

FillRule = _pgmagick.FillRule

FilterTypes = _pgmagick.FilterTypes

class Geometry(_pgmagick.Geometry):
    pass

GravityType = _pgmagick.GravityType

class Image(_pgmagick.Image):
    pass

ImageType = _pgmagick.ImageType

InterlaceType = _pgmagick.InterlaceType

LineCap = _pgmagick.LineCap

LineJoin = _pgmagick.LineJoin

class Montage(_pgmagick.Montage):
    pass

class MontageFramed(_pgmagick.MontageFramed):
    pass

NoiseType = _pgmagick.NoiseType

if hasattr(_pgmagick, "OrientationType"):
    OrientationType = _pgmagick.OrientationType

PaintMethod = _pgmagick.PaintMethod

class PathArcAbs(_pgmagick.PathArcAbs):
    pass

class PathArcArgs(_pgmagick.PathArcArgs):
    pass

class PathArcArgsList(_pgmagick.PathArcArgsList):
    pass

class PathArcRel(_pgmagick.PathArcRel):
    pass

class PathClosePath(_pgmagick.PathClosePath):
    pass

class PathCurveToArgsList(_pgmagick.PathCurveToArgsList):
    pass

class PathCurvetoAbs(_pgmagick.PathCurvetoAbs):
    pass

class PathCurvetoArgs(_pgmagick.PathCurvetoArgs):
    pass

class PathCurvetoRel(_pgmagick.PathCurvetoRel):
    pass

class PathLinetoAbs(_pgmagick.PathLinetoAbs):
    pass

class PathLinetoHorizontalAbs(_pgmagick.PathLinetoHorizontalAbs):
    pass

class PathLinetoHorizontalRel(_pgmagick.PathLinetoHorizontalRel):
    pass

class PathLinetoRel(_pgmagick.PathLinetoRel):
    pass

class PathLinetoVerticalAbs(_pgmagick.PathLinetoVerticalAbs):
    pass

class PathLinetoVerticalRel(_pgmagick.PathLinetoVerticalRel):
    pass

class PathMovetoAbs(_pgmagick.PathMovetoAbs):
    pass

class PathMovetoRel(_pgmagick.PathMovetoRel):
    pass

class PathQuadraticCurvetoAbs(_pgmagick.PathQuadraticCurvetoAbs):
    pass

class PathQuadraticCurvetoArgs(_pgmagick.PathQuadraticCurvetoArgs):
    pass

class PathQuadraticCurvetoArgsList(_pgmagick.PathQuadraticCurvetoArgsList):
    pass

class PathQuadraticCurvetoRel(_pgmagick.PathQuadraticCurvetoRel):
    pass

class PathSmoothCurvetoAbs(_pgmagick.PathSmoothCurvetoAbs):
    pass

class PathSmoothCurvetoRel(_pgmagick.PathSmoothCurvetoRel):
    pass

class PathSmoothQuadraticCurvetoAbs(_pgmagick.PathSmoothQuadraticCurvetoAbs):
    pass

class PathSmoothQuadraticCurvetoRel(_pgmagick.PathSmoothQuadraticCurvetoRel):
    pass

class Pixels(_pgmagick.Pixels):
    pass

QuantumOperator = _pgmagick.QuantumOperator

QuantumType = _pgmagick.QuantumType

RenderingIntent = _pgmagick.RenderingIntent

ResolutionType = _pgmagick.ResolutionType

StorageType = _pgmagick.StorageType

StretchType = _pgmagick.StretchType

StyleType = _pgmagick.StyleType

class TypeMetric(_pgmagick.TypeMetric):
    pass

class VPath(_pgmagick.VPath):
    pass

class VPathList(_pgmagick.VPathList):
    pass




import sys
import unittest
sys.path.append('../')
sys.path.append('./')
from pgmagick import Image, Geometry, Color, LineJoin


class TestImage(unittest.TestCase):

    def test_noarg_init(self):
        im = Image()
        self.assertEqual(type(im), Image)

    def test_fromfile_init_error(self):
        self.assertRaises(RuntimeError, Image, "xXxX.jpg")

    def test_size_and_color_init(self):
        im = Image(Geometry(300, 200), Color('transparent'))
        size = im.size()
        self.assertEqual(300, size.width())
        self.assertEqual(200, size.height())

    def test_stroke_linejoin(self):
        im = Image(Geometry(300, 200), Color('transparent'))
        im.strokeLineJoin(LineJoin.MiterJoin)
        im.strokeLineJoin(LineJoin.RoundJoin)
        im.strokeLineJoin(LineJoin.BevelJoin)

unittest.main()



from setuptools import setup, find_packages, Extension
from distutils.sysconfig import get_python_inc
import glob
import os
import re
import sys

GMCPP_PC = 'GraphicsMagick++.pc'
include_dirs = [get_python_inc()]
library_dirs = []

search_include_dirs = ['/usr/local/include/GraphicsMagick',
                       '/usr/include/GraphicsMagick/']
search_library_dirs = ['/usr/local/lib64/', '/usr/lib64/',
                       '/usr/local/lib/', '/usr/lib/']

if sys.platform == 'Darwin':
    include_dirs.append('/opt/local/include')
    search_include_dirs.extend(['/opt/local/include/GraphicsMagick',
                                '/opt/local/include'])
    search_library_dirs.append('/opt/local/lib')


def _grep(regex, filename):
    for line in open(filename):
        if re.search(regex, line):
            return line


def get_gmversion_from_devheaders(search_dirs):
    target_api_name = "addNoiseChannel"
    for dirname in search_dirs:
        for root, dirs, files in os.walk(dirname):
            for f in files:
                if f == 'Image.h':
                    if _grep(target_api_name, dirname + 'Magick++/Image.h'):
                        return '1.2.x'


def get_gmversion_from_pc(search_dirs):
    """similar to 'pkg-config --modversion GraphicsMagick++'"""
    search_dirs.append('/usr/local/lib/pkgconfig/')
    search_dirs.append('/usr/lib/pkgconfig/')
    for dirname in search_dirs:
        for root, dirs, files in os.walk(dirname):
            for f in files:
                if f == GMCPP_PC:
                    _tmp = _grep("\Version: ", dirname + GMCPP_PC)
                    return _tmp.split()[1]


def find_file(filename, search_dirs):
    for dirname in search_dirs:
        for root, dirs, files in os.walk(dirname):
            for f in files:
                if filename in f:
                    return dirname
            for d in dirs:
                if filename in d:
                    return dirname
            if filename in root:
                return dirname
    raise Exception(filename + " not found")

include_dirs.append(find_file('Magick++', search_include_dirs))
library_dirs.append(find_file('libGraphicsMagick', search_library_dirs))

ext_compile_args = []
gm_version = get_gmversion_from_pc(search_include_dirs)
if not gm_version:
    gm_version = get_gmversion_from_devheaders(include_dirs)
if gm_version:
    gm_version = gm_version.split('.')
    if gm_version[0] == str(1) and gm_version[1] == str(1):    # version 1.1.x
        ext_compile_args = ["-DGM_1_1_x"]
    print "GraphicsMagick++ version: %s.%s.%s" % (gm_version[0],
                                                  gm_version[1],
                                                  gm_version[2])

setup(name='pgmagick',
    version="0.3.4",
    description="Yet Another Python wrapper for GraphicsMagick",
    long_description=open('README.rst').read(),
    author='Hideo Hattori',
    author_email='hhatto.jp@gmail.com',
    url='http://bitbucket.org/hhatto/pgmagick',
    license='MIT',
    platforms='Linux',
    packages=find_packages(),
    ext_modules=[
        Extension('pgmagick._pgmagick',
                  sources=glob.glob('./src/*.cpp'),
                  include_dirs=include_dirs,
                  library_dirs=library_dirs,
                  libraries=['boost_python', 'GraphicsMagick++'],
                  extra_compile_args=ext_compile_args,
                 )],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: POSIX :: Linux',
        'Programming Language :: C++',
        'Programming Language :: Python',
        'Topic :: Multimedia :: Graphics'],
    keywords="GraphicsMagick graphics visualize boost image",
)
